==> kartothek-3.2.0.tar.gz: kartothek-3.2.0/kartothek/io/dask/_utils.py <==
# -*- coding: utf-8 -*-


import warnings

import pandas as pd
from dask import delayed

CATEGORICAL_EFFICIENCY_WARN_LIMIT = 100000


def _identity():
    def _id(x):
        return x

    return _id


def _get_data(mp, table=None):
    """
    Task to avoid serialization of lambdas
    """
    if table:
        return mp.data[table]
    else:
        return mp.data


def _cast_categorical_to_index_cat(df, categories):
    try:
        return df.astype(categories)
    except ValueError as verr:
        # Should be fixed by pandas>=0.24.0
        if "buffer source array is read-only" in str(verr):
            new_cols = {}
            for cat in categories:
                new_cols[cat] = df[cat].astype(df[cat].dtype.categories.dtype)
                new_cols[cat] = new_cols[cat].astype(categories[cat])
            return df.assign(**new_cols)
        raise


def _construct_categorical(column, dataset_metadata_factory):
    dataset_metadata = dataset_metadata_factory.load_index(column)
    values = dataset_metadata.indices[column].index_dct.keys()
    if len(values) > CATEGORICAL_EFFICIENCY_WARN_LIMIT:
        warnings.warn(
            "Column {} has {} distinct values, reading as categorical may increase memory consumption.",
            column,
            len(values),
        )
    return pd.api.types.CategoricalDtype(values)


def _maybe_get_categoricals_from_index(dataset_metadata_factory, categoricals):
    """
    In case a categorical is requested for a column we have an index on,
    construct the categorical from the index
    """
    categoricals_from_index = {}
    if categoricals:
        for table, table_cat in categoricals.items():
            if not table_cat:
                continue
            categoricals_from_index[table] = {}
            for cat in table_cat:
                if cat in dataset_metadata_factory.indices:
                    cat_dtype = _construct_categorical(cat, dataset_metadata_factory)
                    categoricals_from_index[table][cat] = cat_dtype
    return categoricals_from_index


def map_delayed(mps, func, *args, **kwargs):
    return [delayed(func)(mp, *args, **kwargs) for mp in mps]

==> kartothek-3.6.2.tar.gz: kartothek-3.6.2/kartothek/io/dask/_utils.py <==
# -*- coding: utf-8 -*-


import warnings

import pandas as pd
from dask import delayed

CATEGORICAL_EFFICIENCY_WARN_LIMIT = 100000


def _identity():
    def _id(x):
        return x

    return _id


def _get_data(mp, table=None):
    """
    Task to avoid serialization of lambdas
    """
    if table:
        return mp.data[table]
    else:
        return mp.data


def _cast_categorical_to_index_cat(df, categories):
    try:
        return df.astype(categories)
    except ValueError as verr:
        # Should be fixed by pandas>=0.24.0
        if "buffer source array is read-only" in str(verr):
            new_cols = {}
            for cat in categories:
                new_cols[cat] = df[cat].astype(df[cat].dtype.categories.dtype)
                new_cols[cat] = new_cols[cat].astype(categories[cat])
            return df.assign(**new_cols)
        raise


def _construct_categorical(column, dataset_metadata_factory):
    dataset_metadata = dataset_metadata_factory.load_index(column)
    values = dataset_metadata.indices[column].index_dct.keys()
    if len(values) > CATEGORICAL_EFFICIENCY_WARN_LIMIT:
        warnings.warn(
            "Column {} has {} distinct values, reading as categorical may increase memory consumption.",
            column,
            len(values),
        )
    return pd.api.types.CategoricalDtype(values, ordered=False)


def _maybe_get_categoricals_from_index(dataset_metadata_factory, categoricals):
    """
    In case a categorical is requested for a column we have an index on,
    construct the categorical from the index
    """
    categoricals_from_index = {}
    if categoricals:
        for table, table_cat in categoricals.items():
            if not table_cat:
                continue
            categoricals_from_index[table] = {}
            for cat in table_cat:
                if cat in dataset_metadata_factory.indices:
                    cat_dtype = _construct_categorical(cat, dataset_metadata_factory)
                    categoricals_from_index[table][cat] = cat_dtype
    return categoricals_from_index


def map_delayed(mps, func, *args, **kwargs):
    return [delayed(func)(mp, *args, **kwargs) for mp in mps]
